/*
*  Name:         test-1.uesql.cc
*  Author:       Rafael Jesus Alcantara Perez
*  Contributor:  Francisco Vides Fernandez
*  Summary:      Test for UESQL
*  Date:         $Date: 2002/08/19 12:59:57 $
*  Revision:     $Revision: 1.5 $
*
*  Copyright (C) 1996-2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/config_processor.hh>
#include <mpcl/string.hh>
#include <mpcl/test.h>


//
//  L O C A L   V A R I A B L E S
//

static uesql::TSqlstateDescriptor   _tSqlstateDescriptor;
  

//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::TString _GetErrorMessage (void)
{

  using mpcl::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesql::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


static int _TestCursors (void)
{

  using std::cerr;
  using std::endl;
  using uesql::TIdentifier;
  using uesql::TInteger;
  using uesql::TString;
  
  TString       yVar0;
  TString       yVar1;
  TString       yVar2;
  TInteger      tZero (0L);
  bool          gEmpty       = true;
  TIdentifier   yAttribName  = "WORK_CODE";
  size_t        zRecordCount = 0;

  TEST_INIT ("test for cursors");

  extern "UESQL::default"
  {
    whenever sqlerror  goto error;
    whenever not found goto loop_end;
    declare MYCURSOR cursor for
      select NAME, HOURS
        from WORKERS
        where ( NAME = 'nobody' );
    open MYCURSOR;
  }
  while ( true )
  {
    extern "UESQL::default"
    {
      fetch from MYCURSOR into :yVar0, :yVar1;
    }
    gEmpty = false;
  }

loop_end:
  extern "UESQL::default" 
  {
    close MYCURSOR;
  }
  if ( gEmpty )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }

  extern "UESQL::default"
  {
    --
    --  WHENEVER
    --
    whenever sqlerror  goto error;
    whenever not found goto loop_end_2;

    --
    --  DECLARE CURSOR/OPEN/WHENEVER
    --
    declare MYCURSOR cursor for select :yAttribName, NAME, HOURS from WORKERS;
    open MYCURSOR;
  }
    
  if ( hptTestTargetFile )
  {
    fprintf (hptTestTargetFile, "WORK_CODE NAME                                     HOURS\n");
    fprintf (hptTestTargetFile, "========= ======================================== =====\n");
  }
  extern "UESQL::default"
  {
    fetch absolute :tZero from MYCURSOR into :yVar0, :yVar1, :yVar2;
  }
  while ( true )
  {
    extern "UESQL::default"
    {
      fetch from MYCURSOR into :yVar0, :yVar1, :yVar2;
    }
    if ( hptTestTargetFile )
    {
      fprintf (hptTestTargetFile, "%9s %40s %5s\n", yVar0.c_str(), yVar1.c_str(), yVar2.c_str());
    }
    ++zRecordCount;
  }

loop_end_2:
  extern "UESQL::default"
  {
    whenever not found goto loop_end_3;
    fetch absolute :tZero from MYCURSOR into :yVar0, :yVar1, :yVar2;
  }

loop_end_3:
  extern "UESQL::default"
  {
    --
    --  CLOSE CURSOR
    --
    close MYCURSOR;
  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesql::SQLCODE);  

end:
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;
  
}  // _TestCursors()


static int _TestSqlTypes (void)
{

  TEST_INIT ("test for SQL types");

  using std::cerr;
  using std::endl;
  using std::size_t;
  using uesql::TDate;
  using uesql::TInteger;
  using uesql::TReal;
  using uesql::TString;

  TDate      tSqlDate;
  TInteger   tSqlInteger;
  TReal      tSqlReal;
  TString    ySqlString;
  size_t     zRecordCount = 0;

  TEST_NUMBERS (1,     TInteger ("1"));
  TEST_NUMBERS (0,     TInteger ("0"));
  TEST_NUMBERS (10000, TInteger ("10000"));
  TEST_NUMBERS (10001, TInteger (10001));
  TEST_NUMBERS (true,  TDate    ((const char*) NULL).isNull());
  TEST_NUMBERS (true,  TInteger ((const char*) NULL).isNull());
  TEST_NUMBERS (true,  TReal    ((const char*) NULL).isNull());
  TEST_NUMBERS (true,  TString  ((const char*) NULL).isNull());
  TEST_NUMBERS (true,  TDate    ("").isNull());
  TEST_NUMBERS (true,  TInteger ("").isNull());
  TEST_NUMBERS (true,  TReal    ("").isNull());
  TEST_NUMBERS (false, TString  ("").isNull());
  TEST_NUMBERS (true,  TDate().isNull());
  TEST_NUMBERS (true,  TInteger().isNull());
  TEST_NUMBERS (true,  TReal().isNull());
  TEST_NUMBERS (true,  TString().isNull());
  tSqlInteger.getBuffer (12);
  tSqlInteger.getBuffer (12);
  tSqlInteger.getBuffer (12);
  tSqlInteger.set ("75000");
  TEST_NUMBERS (75000, tSqlInteger);
  extern "UESQL::default" 
  {
    whenever sqlerror  goto error;
    whenever not found goto loop_end;
    create table TABLE_1
    (
      CODE       character (3) not null ,
      BIRTH_DATE date                   ,
      SALARY     integer                ,
      PERCENTAGE double precision
    );
    insert into TABLE_1 values ('000', '1999-12-31', 12000, 25.75);
    insert into TABLE_1 values ('001', '1970-01-01', 75000, 30);
    insert into TABLE_1 values ('002', '2000-01-01', 11500, 99.99);
    insert into TABLE_1 values ( '003'                    ,
                                 :TDate (2000, 12, 31) ,
                                 :TInteger (7500)      ,
                                 :TReal (33.33)        );
    insert into TABLE_1 values ( '004'                    ,
                                 :TDate (2001,  1,  1) ,
                                 :TInteger (6500)      ,
                                 :TReal (0.0)          );
    declare MYCURSOR cursor for
      select CODE, BIRTH_DATE, SALARY, PERCENTAGE from TABLE_1;
    open MYCURSOR;
  }

  //
  //  Writes the example table.
  //
  if ( hptTestTargetFile )
  {
    fprintf (hptTestTargetFile, "%s\n", "CODE BIRTH_DATE   SALARY PERCENTAGE");
    fprintf (hptTestTargetFile, "%s\n", "==== ============ ====== ==========");
  }
  while ( true )
  {
    extern "UESQL::default"
    {
      fetch from MYCURSOR into :ySqlString, :tSqlDate, :tSqlInteger, :tSqlReal;
    }
    if ( hptTestTargetFile )
    {
      fprintf ( hptTestTargetFile          ,
                "%3s  %12s %6ld %10.03f\n" ,
                ySqlString.c_str()         ,
                tSqlDate.get().c_str()     ,
                (long int) tSqlInteger     ,
                (double) tSqlReal          );
    }
    ++zRecordCount;
  }

loop_end:
  extern "UESQL::default" 
  {
    close MYCURSOR;
    drop table TABLE_1 cascade;
  }
  if ( zRecordCount == 5 )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesql::SQLCODE);  

end:  
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // _TestSqlTypes()


/**
*  Main.
*
*  Parameters:
*
*    {-o, --dbms-server-name}=DBMS-SERVER-NAME
*    {-u, --dbms-user-name}=DBMS-USER-NAME
*    {-p, --dbms-user-password}=PASSWORD
*/
int main (int argc, const char* argv[])
{

  using mpcl::Format;
  using mpcl::TConfigProcessor;
  using std::cerr;
  using std::endl;
  using uesql::ComposeSchemaLocation;
  using uesql::ComposeUserLogin;
  using uesql::TIdentifier;
  using uesql::TInteger;
  using uesql::TString;
  
  TEST_INIT ("test-1");
  TEST_START_WATCH;

  TInteger           tInteger;
  TString            yVar0;
  TString            yVar1;
  TString            yVar2;
  TIdentifier        yCursorName;
  TString            yDbmsUserLogin;
  TString            yDbmsSchemaLocation;
  TString            yDbmsServerName;
  TConfigProcessor   tConfigProcessor (argc, argv);
  TString            yCodeValue ("002");
  const char*        pcDbmsServerName = getenv ("DBMS_SERVER_NAME");
  const char*        pcDbmsUserName   = getenv ("DBMS_USER_NAME");
  TString*           pyDbmsUserLogin  = &yDbmsUserLogin;

  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    if ( !pcDbmsUserName )
    {
      pcDbmsUserName = getenv ("USER");
      if ( !pcDbmsUserName )
      {
        pcDbmsUserName = "";
      }
    }
    
    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name",   "o", pcDbmsServerName);
    tConfigProcessor.addOption ("dbms-user-name",     "u", pcDbmsUserName);
    tConfigProcessor.addOption ("dbms-user-password", "p", pcDbmsUserName);
    tConfigProcessor.processOptions();
    
    yDbmsServerName     = tConfigProcessor ["dbms-server-name"].getValue();
    yDbmsUserLogin      = tConfigProcessor ["dbms-user-name"].getValue();
    yDbmsSchemaLocation = ComposeSchemaLocation
                          (
                            yDbmsServerName                                ,
                            tConfigProcessor ["dbms-user-name"].getValue()
                          );

    //
    //  If there is a password then appends it to the user-login string.
    //
    if ( !tConfigProcessor ["dbms-user-password"].getValue().empty() )
    {
      yDbmsUserLogin = ComposeUserLogin
                       (
                         yDbmsUserLogin                                     ,
                         tConfigProcessor ["dbms-user-password"].getValue() 
                       );
    }
  }
  catch (const mpcl::TBaseException& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }

  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN      = %s", yDbmsUserLogin.c_str());
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::tLog.also (cerr);
  }
  if ( !yDbmsServerName.empty() && !yDbmsUserLogin.empty() )
  {
    extern "UESQL::default" 
    {
      --
      --  WHENEVER
      --
      whenever sqlerror goto error;
      whenever not found goto not_found;
      
      --
      --  CONNECT TO
      --
      connect to :yDbmsSchemaLocation user :yDbmsUserLogin;

      --
      --  CREATE TABLE
      --
      create table JOBS
      (
        CODE         character (3) not null ,
        WORKER_COUNT integer
      );

      create table WORKERS
      (
        WORK_CODE character (3)  not null ,
        NAME      character (40) not null ,
        HOURS     decimal (5)
      );

      --
      --  ALTER TABLE
      --
      alter table WORKERS add column STATE character (40);
      alter table WORKERS add column PHONE_NUMBER integer;

      --
      --  DELETE
      --
      delete from JOBS where ( CODE = :yCodeValue );
      delete from JOBS where ( CODE = '001' );
      delete from WORKERS;

      --
      --  INSERT
      --
      insert into JOBS values ('001', 100);
      insert into JOBS values ('002', 250);
      insert into WORKERS values ('001', 'myname',   40,   'Andalucia',   5550000);
      insert into WORKERS values ('001', 'yourname', 35,   'Extremadura', 5550001);
      insert into WORKERS values ('001', 'hisname',  25,   'Catalunya',   5550002);
      insert into WORKERS values ('002', 'john',     35,   'New England', 5550003);
      insert into WORKERS values ('002', 'james',    35,   'Washington',  5550004);
      insert into WORKERS values ('002', 'william',  NULL, 'Missouri',    5550005);
      insert into WORKERS values ('002', 'peter',    NULL, 'Arkansas',    5550006);
      insert into WORKERS select WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER from WORKERS where ( NAME = 'myname' );
      insert into WORKERS select WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER from WORKERS;

      --
      --  WHENEVER
      --
      whenever sqlerror goto cardinality_check;

      --
      --  SELECT for a single row.
      --
      --    Cardinality violation, so ( SQLSTATE == '21000' ).
      --
      select HOURS into :yVar1 from WORKERS, JOBS where ( WORK_CODE = CODE );
    }
    TEST_FAIL;

cardinality_check:
    TEST_STRINGS (uesql::SQLSTATE.c_str(), "21000");
    extern "UESQL::default" 
    {
      --
      --  WHENEVER
      --
      whenever sqlerror goto error;

      --
      --  SELECT for a single row.
      --
      select count (*) into :tInteger from WORKERS where ( HOURS is null );

      --
      --  DECLARE CURSOR/OPEN/FETCH/CLOSE
      --
      declare MYCURSOR cursor for select WORK_CODE, HOURS from WORKERS where ( HOURS is null );
      open MYCURSOR;
      fetch from MYCURSOR into :yVar0, :yVar1;
      close MYCURSOR;
    }
    TEST_NUMBERS (tInteger,          4);
    TEST_NUMBERS (tInteger.isNull(), false);
    TEST_NUMBERS (yVar0.isNull(),    false);
    TEST_NUMBERS (yVar1.isNull(),    true);
    
    yCursorName = "MYCURSOR";
    extern "UESQL::default" 
    {
      --
      --  WHENEVER
      --
      whenever sqlerror goto error;

      --
      --  SELECT for a single row.
      --
      select count (*) into :yVar2 from WORKERS where name = 'myname';

      --
      --  UPDATE
      --
      update WORKERS
        set PHONE_NUMBER = WORKERS.PHONE_NUMBER
        where ( ( WORKERS.NAME = 'myname' ) and ( WORKERS.NAME = NAME ) );
    }
    TEST_FUNCTION (_TestCursors());
    TEST_FUNCTION (_TestSqlTypes());
    extern "UESQL::default" 
    {
      --
      --  DECLARE CURSOR/OPEN/FETCH/CLOSE
      --
      declare MYCURSOR cursor for select CODE from JOBS where CODE = '001';
      open MYCURSOR;
      fetch from :yCursorName into :yVar0;
      close MYCURSOR;
    }
    TEST_STRINGS (uesql::SQLSTATE.c_str(), "00000");
    TEST_STRINGS (yVar2.c_str(),           "4");
    TEST_STRINGS (yVar0.c_str(),           "001");
    extern "UESQL::default" 
    {
      --
      --  REVOKE
      --
      revoke all privileges on JOBS from public cascade;
      revoke all privileges on WORKERS from public restrict;

      --
      --  GRANT
      --
      grant select on JOBS    to public;
      grant select on WORKERS to public;
      grant delete on WORKERS to public;

      --
      --  CREATE VIEW
      --
      create view VIEW1 as select * from JOBS;

      --
      --  DROP VIEW
      --
      drop view VIEW1 cascade;

      --
      --  DROP TABLE
      --
      drop table JOBS cascade;

      --
      --  DISCONNECT
      --
      disconnect current;
    }

    extern "UESQL::default"
    {
      --
      --  CONNECT TO
      --
      connect to :yDbmsSchemaLocation as 'myconnection' user :*pyDbmsUserLogin;

      --
      --  DROP TABLE
      --
      drop table WORKERS cascade;
    }
    goto end;

  error:
    cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
    cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
    cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (uesql::SQLCODE);
    goto end;

  not_found:
    cerr << "Search failed\n";
    TEST_FAIL_WITH_CODE (2);
    goto end;
  }

end:
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::tLog.erase (cerr);
  }
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH ("UESQL benchmark");
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
